--
-- DigitalBillboards
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	v1.0 - 2016-04-16 - Initial implementation
-- 

DigitalBillboard = {};

local DigitalBillboard_mt = Class(DigitalBillboard, Object);
InitObjectClass(DigitalBillboard, "DigitalBillboard");

function DigitalBillboard.onCreate(id)
    local instance = DigitalBillboard:new(g_server ~= nil, g_client ~= nil);
	if instance:load(id) then
		g_currentMission:addOnCreateLoadedObject(instance);
		instance:register(true);
	else
		instance:delete();
	end;
end;
g_onCreateUtil.addOnCreateFunction("DigitalBillboard", DigitalBillboard.onCreate);

function DigitalBillboard:new(isServer, isClient)
    local self = Object:new(isServer, isClient, DigitalBillboard_mt);
    self.className = "DigitalBillboard";
	
	self.currentImage = 1;
	self.maxNumImages = 0;
	self.images = {};
	
	self.currentTime = 0;
	self.swapIntervall = 60; -- seconds
	
    return self;
end;

function DigitalBillboard:load(id)
	self.maxNumImages = getNumOfChildren(id);
	if self.maxNumImages > 0 then
		for i = 1, self.maxNumImages do
			table.insert(self.images, getChildAt(id, i - 1));
		end;
	else
		print("DigitalBillboard - Error: There are no valid 'images' we can use.");
		return false;
	end;
	
	self.swapIntervall = Utils.getNoNil(getUserAttribute(id, "swapIntervall"), self.swapIntervall) * 1000;
	
	if self.maxNumImages > 1 and Utils.getNoNil(getUserAttribute(id, "shuffleImages"), false) then
		function shuffle(images)
			for i = table.getn(images), 2, -1 do
				local j = math.random(1, i);
				images[i], images[j] = images[j], images[i];
			end;
			
			return images;
		end;

		self.images = shuffle(self.images);
	end;

	self:setBillboardImage(self.currentImage);
	
	return true;
end;

function DigitalBillboard:delete()
end;

function DigitalBillboard:readStream(streamId, connection)
	if connection:getIsServer() and self.maxNumImages > 1 then
		self.currentTime = streamReadFloat32(streamId);
		self.currentImage = streamReadInt8(streamId);
		
		self:setBillboardImage(self.currentImage);
	end;
end;

function DigitalBillboard:writeStream(streamId, connection)	
	if not connection:getIsServer() and self.maxNumImages > 1 then
		streamWriteFloat32(streamId, self.currentTime);
		streamWriteInt8(streamId, self.currentImage);
	end;
end;

function DigitalBillboard:update(dt)
end;

function DigitalBillboard:updateTick(dt)
	if self.maxNumImages > 1 then -- There is no point in swapping if we dont have enough images!
		if self.currentTime > self.swapIntervall then
			self:setNewSwapId(self.currentImage + 1);
			self:setBillboardImage(self.currentImage);
			
			self.currentTime = 0;
		else
			self.currentTime = self.currentTime + dt;
		end;
	end;
end;

function DigitalBillboard:setNewSwapId(id)
	if id > self.maxNumImages then
		id = 1;
	end;
	
	self.currentImage = id;
end;

function DigitalBillboard:setBillboardImage(id)
	for i, node in ipairs(self.images) do
		setVisibility(node, i == id);
	end;
end;